


#include <board.h>
#include <tc/tc.h>
#include <aic/aic.h>
#include <rtt/rtt.h>
#include <pit/pit.h>

#include <utility/trace.h>
#include <utility/assert.h>
#include "clock-arch.h"

#ifndef AT91C_ID_TC0
#if defined(AT91C_ID_TC012)
    #define AT91C_ID_TC0 AT91C_ID_TC012
#elif defined(AT91C_ID_TC)
    #define AT91C_ID_TC0 AT91C_ID_TC
#else
    #error Pb define ID_TC
#endif
#endif

/// Define AT91C_BASE_RTTC if not already done
#if !defined(AT91C_BASE_RTTC)
    #define AT91C_BASE_RTTC     AT91C_BASE_RTTC0
#endif


#define BOARD_MCK_MHZ ((BOARD_MCK)/1000000)
/*-----------------------------------------------------------------------------
 *        Internal variables
 *-----------------------------------------------------------------------------*/
/**
* \external function called every \link #clockTick\endlink.
* which will invoke cocoOS scheduler.
*/
extern void os_tick( void );

/** clock tick count */
static unsigned int clockTick;

/** unsigned int holding the current timestamp*/
unsigned int timestamp;


/*-----------------------------------------------------------------------------
 *        Local functions
 *-----------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/** \internal
 * Timer Counter 0 interrupt handler.
 * \param void
 * \return void
 */
/*---------------------------------------------------------------------------*/
static void ISR_Tc0(void)
{
    /* Acknowledge interrupt*/
    AT91C_BASE_TC0->TC_SR;

    /*Increase tick*/

    if(! (clockTick % OS_TICK) ){
        /*If it does not div OS_TICK it will not div CLOCK_CONF_SECOND*/
        if(! (clockTick % CLOCK_CONF_SECOND) ){
            timestamp++;
        }
        os_tick();
    }

    clockTick ++;
}


/*---------------------------------------------------------------------------*/
 /**\internal
  * Interrupt handler for the RTT. Displays the current time on the DBGU.
  * \param void
  * \return void
  */
/*---------------------------------------------------------------------------*/
static void ISR_Sys(void)
{
    if(PIT_GetStatus()){
        /*Acknowledge interrupt*/
        PIT_GetPIVR();

        /*Increase tick*/
        if(! (clockTick % OS_TICK) ){
            /*If it does not div OS_TICK it will not div CLOCK_CONF_SECOND*/
            if(! (clockTick % CLOCK_CONF_SECOND) ){
                timestamp++;
            }
            os_tick();
        }

        clockTick ++;

    }

}

/*---------------------------------------------------------------------------*/
 /**\internal
  * Configures the RTT to generate a one second tick, which triggers the RTTINC
  *  interrupt.
  * \param void
  * \return void
  */
/*---------------------------------------------------------------------------*/
inline  static void ConfigureRtt(void)
{
    unsigned int previousTime;
    TRACE_INFO("Rtt init ...\n\r");
    // Configure RTT for a 1 second tick interrupt
    RTT_SetPrescaler(AT91C_BASE_RTTC, 32768);
    previousTime = RTT_GetTime(AT91C_BASE_RTTC);
    while (previousTime == RTT_GetTime(AT91C_BASE_RTTC));

    /* Enable RTT interrupt*/
    AIC_ConfigureIT(AT91C_ID_SYS, 0, ISR_Sys);
    AIC_EnableIT(AT91C_ID_SYS);

    /*Enable the RTT*/
    RTT_EnableIT(AT91C_BASE_RTTC, AT91C_RTTC_RTTINCIEN);

}
/*---------------------------------------------------------------------------*/
 /** \internal
  *  Configures the PIT.
  * \param void
  * \return void
  */
/*---------------------------------------------------------------------------*/
inline static void ConfigurePit(void){
    TRACE_INFO("Pit init ...\n\r");
    /*Initialize the PIT*/
    PIT_Init(CLOCK_CONF_SECOND, BOARD_MCK_MHZ);
    /*Enables the PIT*/
    PIT_Enable();

    /* Enable RTT interrupt*/
    AIC_ConfigureIT(AT91C_ID_SYS, 0, ISR_Sys);
    AIC_EnableIT(AT91C_ID_SYS);

    /*Enable the PIT*/
    PIT_EnableIT();
}

/*---------------------------------------------------------------------------*/
 /**\internal
  * Configures the TC0.
  * \param void
  * \return void
  */
/*---------------------------------------------------------------------------*/
inline  static void ConfigureTc0(void){
    unsigned int div, tcclks;
    TRACE_INFO("TC0 init ...\n\r");

    /* Enable TC0*/
    AT91C_BASE_PMC->PMC_PCER = 1 << AT91C_ID_TC0;
    ASSERT(TC_FindMckDivisor(CLOCK_CONF_SECOND, BOARD_MCK, &div, &tcclks),
           "F: Fail to generate the desired frequency %d\n\r",
           CLOCK_CONF_SECOND);
    TC_Configure(AT91C_BASE_TC0, tcclks | AT91C_TC_WAVE | AT91C_TC_WAVESEL_UP_AUTO);
    AT91C_BASE_TC0->TC_RC = BOARD_MCK / (CLOCK_CONF_SECOND * div);

    /* Configure TC0 interrupt*/
    AIC_DisableIT(AT91C_ID_TC0);
    AIC_ConfigureIT(AT91C_ID_TC0, 0, ISR_Tc0);
    AT91C_BASE_TC0->TC_IER = AT91C_TC_CPCS;
    AIC_EnableIT(AT91C_ID_TC0);

    /* Clear tick value*/
    clockTick = 0;

    /* Start timer*/
    TC_Start(AT91C_BASE_TC0);
}


/*---------------------------------------------------------------------------*/
/**
 * Initialize for clock time operation
 * \param void
 * \return void
 */
/*---------------------------------------------------------------------------*/

void clock_init(void)
{
    ConfigurePit();
    /*
	ConfigureRtt();
    ConfigureTc0();
	
	*/
}



/*---------------------------------------------------------------------------*/
/**
 * Read the clock time (ms)
 * \param void
 * \return clock time in (ms)
 */
/*---------------------------------------------------------------------------*/
clock_time_t clock_time(void) {
    return clockTick;
}
/*---------------------------------------------------------------------------*/
/** @} */
